<?php
/**
 * LICENCIA
 *
 * Esta obra está bajo una licencia Reconocimiento-No comercial-Sin obras
 * derivadas 2.5 de Creative Commons. Para ver una copia de esta licencia,
 * visite http://creativecommons.org/licenses/by-nc-nd/4.0/ o
 * envie una carta a Creative Commons, 171 Second Street, Suite 300,
 * San Francisco, California 94105, USA.
 *
 * @author      iuttu
 * @link        http://www.iuttu.com
 * @copyright   (c) 2015 iuttu
 */
namespace iuttu\cache;

DEFINE('CACHE_BASE_DIR', __DIR__ . '/cache');

global $cached_page_id;
$cached_page_id = get_page_id();

/**
 * Devuelve el nombre del fichero que guarda la caché
 */
function get_page_filename(){
	global $cached_page_id;
	return CACHE_BASE_DIR . DIRECTORY_SEPARATOR . $cached_page_id . '.html';
}

/**
 * Devuelve el nombre del fichero que guarda la caducidad de la caché
 */
function get_page_control_filename(){
	global $cached_page_id;
	return CACHE_BASE_DIR . DIRECTORY_SEPARATOR . $cached_page_id . '.time';
}

/**
 * Devuelve la url actual
 */
function get_current_url(){
    $current_url = (@$_SERVER["HTTPS"] == "on") ? "https://" : "http://";
    $current_url .= $_SERVER["SERVER_NAME"];

    if($_SERVER["SERVER_PORT"] != "80" && $_SERVER["SERVER_PORT"] != "443"){
        $current_url .= ":".$_SERVER["SERVER_PORT"];
    }

    $current_url .= $_SERVER["REQUEST_URI"];

    $current_url = str_replace('?no_cache=true', '', $current_url);

    return $current_url;
}

/**
 * Recuperamos el id de la página a partir de la url
 */
function get_page_id(){
    return md5(get_current_url());
}

/**
 * Cargamos la página de disco
 */
function load(){
	$file_name = get_page_filename();
	if (file_exists($file_name)) {
		// existe, está caducado?
		//$valid_till = file_get_contents(get_page_control_filename());

		$generated_time = filemtime($file_name);

		if($generated_time && (time() - $generated_time) <= (60 * 60 * 24 * 2)) {
			return file_get_contents($file_name);
		}else{
			return null;
		}

	} else {
		return null;
	}
}

/**
 * Recuperamos la página
 */
function get_page(){
	if(!is_user_logged_in()){
		if (($_SERVER['REQUEST_METHOD'] !== 'POST') && (!isset($_GET['no_cache'])) && should_be_cached()) {
			$content = load();
			if (!is_null($content)) {

				echo parse_cached_content($content);
				die('<!-- page cached -->');
			} else {
				ob_start();
			}
		} else {
			if (isset($_GET['no_cache']) /*|| strpos(get_current_url(), '/ca/') != false*/) {
				ob_start();
			}
		}
	}
}

/**
 * Parsea el contenido cacheado y lo devuelve adecuado a la visita actual
 */
function parse_cached_content($content){
	$start			= 'prebrowser';
	$end			= 'postbrowser';
	$replacement	= ' ' . implode(' ', browser_class()) . ' ';
	$content		= replace_between($content, $start, $end, $replacement);

	if(strpos($content, 'wpcf7_captcha_challenge_captcha') !== false){

		$captcha_dir	= __DIR__ . '/wp-content/uploads/wpcf7_captcha/';
		$images			= glob($captcha_dir . '*.{png}', GLOB_BRACE);
		shuffle($images);

		if($images){

			$pathinfo		= pathinfo($images[0]);
			$new_id			= $pathinfo['filename'];

			$pattern = '<input type="hidden" name="_wpcf7_captcha_challenge_captcha-([0-9]+)" value="([0-9]+)">';
			preg_match_all($pattern, $content, $matches);

			if(!empty($matches[2])){
				foreach($matches[2] as $match){
					$content = str_replace('"' . $match . '"', '"' . $new_id . '"', $content);
					$content = str_replace('wpcf7_captcha/' . $match . '.png', 'wpcf7_captcha/' . $new_id . '.png', $content);
				}
			}
		}
	}

	return $content;
}

/**
 * Comprueba si el usuario ha hecho login mirando la cookie
 */
function is_user_logged_in(){
	$cookies_ids = array_keys($_COOKIE);
	return preg_grep('/wordpress_logged_in(\w+)/i', $cookies_ids);
}

/**
 * Guarda una página en caché
 */
function save_page(){
	global $cached_page_id;
	if(!is_user_logged_in() && ($_SERVER['REQUEST_METHOD'] !== 'POST')){
		$content = ob_get_contents();
		ob_end_clean();
		$strip_content = strip_tags($content);

		if (!empty($strip_content)){
			file_put_contents(get_page_filename(), $content . '</body></html>');
			//file_put_contents(get_page_control_filename(), time() + (60 * 60 * 24 * 2));
		}
		echo $content;
	}
}

/**
 * Indica si la url actual debe cachearse
 */
function should_be_cached(){
	$result = true;

	$current_url = get_current_url();

	if(	strpos($current_url, 'wp-login') !== false ||
		strpos($current_url, 'wp-admin') !== false ||
		//strpos($current_url, '/ca/') !== false ||
		strpos($current_url, '/registro') !== false ||
		strpos($current_url, 'notify') !== false ||
		strpos($current_url, 'products') !== false ||
		strpos($current_url, 'gateway') !== false ||
		strpos($current_url, 'gracias-por-tu-aportacion') !== false ||
		strpos($current_url, '/registration') !== false ||
		strpos($current_url, '/fundacion/colabora-con-la-fundacion/donacion') !== false ||
		strpos($current_url, '/rosas-solidarias-de-sant-jordi') !== false ||
		strpos($current_url, 'ca/roses-solidaries-de-sant-jordi') !== false 

	){
		$result = false;
	}

	return $result;
}

/**
 * Devuelve el class relacionado con el navegador que visita la web
 */
function browser_class(){
	// obtenemos información del navegador
	$userAgent = strtolower($_SERVER['HTTP_USER_AGENT']);

	// Recuperamos el navegador
	$browser = '';
	if (preg_match('/chrome/', $userAgent))
		$browser = 'chrome';
	elseif (preg_match('/iphone/', $userAgent))
		$browser = 'iphone';
	elseif (preg_match('/ipod/', $userAgent))
		$browser = 'ipod';
	elseif (preg_match('/android/', $userAgent))
		$browser = 'android';
	elseif (preg_match('/opera/', $userAgent))
		$browser = 'opera';
	elseif (preg_match('/webkit/', $userAgent))
		$browser = 'safari';
	elseif (preg_match('/msie/', $userAgent))
		$browser = 'msie';
	elseif (preg_match('/firefox/', $userAgent))
		$browser = 'firefox';
	elseif (preg_match('/mozilla/', $userAgent) && !preg_match('/compatible/', $userAgent))
		$browser = 'mozilla';

	$classes[] = $browser;

	// Recuperamos la versión
	switch($browser){
		case 'opera':
		case 'iphone':
			preg_match("#(version)[/ ]?([0-9.]*)#", $userAgent, $matches);
			break;
		default :
			preg_match("#($browser)[/ ]?([0-9.]*)#", $userAgent, $matches);
			break;
	}

	if($matches){
		$version = str_replace('.','_',$matches[2]);
		$minorVersion = floor($version);
		$classes[] = $browser.$version;
		$classes[] = $browser.$minorVersion;
	}

	// Recuperamos el SO
	if (preg_match('/linux/', $userAgent))
		$classes[] = 'linux';
	elseif (preg_match('/ipad/', $userAgent))
		$classes[] = 'ipad';
	elseif (preg_match('/macintosh|mac os x/', $userAgent))
		$classes[] = 'mac';
	elseif (preg_match('/windows|win32/', $userAgent))
		$classes[] = 'windows';

	return $classes;
}

function replace_between($str, $needle_start, $needle_end, $replacement) {
    $pos = strpos($str, $needle_start);
    $start = $pos === false ? 0 : $pos + strlen($needle_start);

    $pos = strpos($str, $needle_end, $start);
    $end = $pos === false ? strlen($str) : $pos;

    return substr_replace($str, $replacement, $start, $end - $start);
}